# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['drf_rfc7807_exceptions']

package_data = \
{'': ['*']}

install_requires = \
['Django>=2.2', 'djangorestframework>=3.0']

setup_kwargs = {
    'name': 'drf-rfc7807-exceptions',
    'version': '0.1.0',
    'description': 'A library for Django Rest Framework returning consistent and easy-to-parse error messages inspired by RFC7807 guidelines.',
    'long_description': '# Django Rest Framework RFC7807 Exceptions\n\n## What is this?\n\nA library for [Django Rest Framework](https://www.django-rest-framework.org/) returning consistent and easy-to-parse error messages.\n\nThis library was built with [RFC7807](https://tools.ietf.org/html/rfc7807) guidelines in mind, but with a small twist: it defines a "problem detail" as a `list` but it still serves as a way to include errors in a predictable and easy-to-parse format for any API consumer.\n\nThis library was designed to be used by anyone, therefore all of the advanced RFC7807 "problem detail" components are optional.\nErrors, on the other hand, are always formatted with RFC7807 keywords and DRF exception data.\n\n**N.B.**\n\n**This is still in the works, and the ability to raise RFC7870-like errors will be added in the near future.**\n\n## Table of Contents\n\n- [Installation](#installation)\n- [Usage](#usage)\n  - [Exception Handler](#exception-handler)\n  - [Example JSON Error Responses](#example-json-error-responses)\n  - [Settings](#settings)\n    - [CAMELIZE](#camelize)\n    - [EXTRA_HANDLERS](#extra_handlers)\n    - [FIELDS_SEPARATOR](#fields_separator)\n- [Testing](#testing)\n- [Support](#support)\n- [Contributing](#contributing)\n\n## Installation\n\nInstall using the command line:\n\n```\npip install drf-rfc7807-exceptions\n```\n\n## Usage\n\n### Exception Handler\n\nAdd `EXCEPTION_HANDLER` in your `REST_FRAMEWORK` settings of your Django project settings file:\n\n```python\nREST_FRAMEWORK = {\n    # ...\n    "EXCEPTION_HANDLER": "drf_rfc7807_exceptions.exception_handler",\n}\n```\n\n### Example JSON Error Responses\n\n#### Field validation errors\n\n```json\n{\n    "title": "Error message.",\n    "invalid_params": [\n        {\n            "name": "field_name",\n            "reason": [\n                "error",\n                ...\n            ]\n        },\n        ...\n    ]\n}\n```\n\n#### Non-fields validation errors\n\n```json\n{\n  "title": "Error message.",\n  "detail": [\n    "error",\n    ...\n  ]\n}\n```\n\n#### Other bad requests with no detail\n\n```json\n{\n  "title": "Error message."\n}\n```\n\n#### RFC7807 errors\n\n_Coming soon..._\n\n## Settings\n\nDefault available settings:\n\n```python\nDRF_RFC7807_EXCEPTIONS = {\n    "CAMELIZE": False,\n    "EXTRA_HANDLERS": [],\n    "FIELDS_SEPARATOR": ".",\n}\n```\n\n- #### CAMELIZE\n\nCamel case support for Django Rest Framework exceptions JSON error responses.\n\nIf `CAMELIZE` is set to `True`:\n\n```json\n{\n  "title": "Error message.",\n  "invalidParams": [\n    {\n      "name": "fieldName",\n      "reason": [\n        "error",\n        ...\n      ]\n    }\n    ...\n  ]\n}\n```\n\n- #### EXTRA_HANDLERS\n\nSupport for exceptions that differ from the standard structure of the Django Rest Framework.\n\nFor instance, you may want to specify you own exception:\n\n```python\nclass AuthenticationFailed(exceptions.AuthenticationFailed):\n    def __init__(self, detail=None, code=None):\n        """\n        Builds a detail dictionary for the error to give more information\n        to API users.\n        """\n        detail_dict = {"detail": self.default_detail, "code": self.default_code}\n\n        if isinstance(detail, dict):\n            detail_dict.update(detail)\n        elif detail is not None:\n            detail_dict["detail"] = detail\n\n        if code is not None:\n            detail_dict["code"] = code\n\n        super().__init__(detail_dict)\n```\n\nUse exception in code:\n\n```python\ndef my_func():\n    raise AuthenticationFailed(\n        {\n            "detail": _("Error message."),\n            "messages": [\n                {\n                    "metadata": "metadata_data",\n                    "type": "type_name",\n                    "message": "error message",\n                }\n            ],\n        }\n    )\n```\n\nThis will result in:\n\n```python\nAuthenticationFailed(\n    {\n        "detail": "Error message.",\n        "messages": [\n            {\n                "metadata": "metadata_data",\n                "type": "type_name",\n                "message": "error message",\n            }\n        ],\n    }\n)\n```\n\nYou can handle this by creating a `handlers.py` file and specifying an handler for your use case:\n\n```python\ndef handle_exc_custom_authentication_failed(exc):\n    from path.to.my.exceptions import AuthenticationFailed\n\n    if isinstance(exc, AuthenticationFailed):\n        try:\n            exc.detail = exc.detail["messages"][0]["message"]\n        except (KeyError, IndexError):\n            exc.detail = exc.detail["detail"]\n\n    return exc\n```\n\nThen add it to the `EXTRA_HANDLERS` list in this package settings:\n\n```python\nDRF_RFC7807_EXCEPTIONS = {\n    "EXTRA_HANDLERS": [\n        "path.to.my.handlers.handle_exc_custom_authentication_failed",\n        # ...\n    ]\n}\n```\n\n- #### FIELDS_SEPARATOR\n\nSupport for nested dicts containing multiple fields to be flattened.\n\nIf `FIELDS_SEPARATOR` is set to `.`:\n\n```python\n{\n    "field1": {\n        "field2": "value"\n    }\n}\n```\n\nWill result in:\n\n```python\n{\n    "field1.field2": "value"\n}\n```\n\n## Testing\n\n### Using tox (recommended)\n\n```\ntox\n```\n\n### Using poetry\n\n##### Install dependencies:\n\n```\nmake install\n```\n\nor\n\n```\npoetry install -v\n```\n\n##### Run tests:\n\n```\nmake test\n```\n\nor\n\n```\npoetry run pytest test_project\n```\n\n## Support\n\nPlease [open an issue](https://github.com/gripep/drf-rfc7807-exceptions/issues/new).\n\n## Contributing\n\nPlease use the [Github Flow](https://guides.github.com/introduction/flow/). In a nutshell, create a branch, commit your code, and open a pull request.\n',
    'author': 'Gian',
    'author_email': '30044863+gripep@users.noreply.github.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/gripep/drf-rfc7807-exceptions',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8.1,<4.0',
}


setup(**setup_kwargs)
